import csv


class FileOpener:
    """To import, open, and read file"""

    @staticmethod
    def openCSV(path, withHeader=False):
        dataList = []
        reader = csv.reader(open(path, "r"))
        if withHeader:
            next(reader)
        for row in reader:
            dataList.append(row)
        return dataList



import pytest

# from database.service import DBService
from base.base_driver import BaseDriver
from ._fixtures.composition import *
from ._fixtures.auth import *
from ._fixtures.option import *
from ._fixtures.param import *


@pytest.fixture(scope="class")
def setup(request, logger, browser, domain, service_account, store):
    logger

    # setup driver
    base = BaseDriver(browser, domain, fullscreen=True)

    # setup service
    # service = DBService(domain)
    # service.start()

    # set requests
    request.cls.base = base
    request.cls.sa = service_account
    request.cls.store = store
    # request.cls.service = service

    yield

    # service.end()
    base.exit()



from dotenv import load_dotenv
import gspread
import os

load_dotenv()


class NamedRanges:
    def __init__(self) -> None:
        self.spreadsheetName = os.getenv("SPREADSHEET_NAME")
        self.sa = gspread.service_account()
        self.ss = self.sa.open(self.spreadsheetName)
        self.ansi = {
            "error": "\033[91m\033[1m",
            "info": "\033[94m",
            "success": "\u001b[32m",
            "warn": "\u001b[33m",
            "reset": "\033[0m",
        }
        self.optX = lambda oriColor, opt: f"[{self.ansi['success']}{opt}{self.ansi[oriColor]}]: {self.ansi['reset']}"
        self.obj = lambda obj: f"'{obj}'"

    def opt(self, oriColor: str, *args) -> str:
        if args:
            printed = "("
            for arg in args:
                printed += f"{self.ansi['success']}{arg}{self.ansi[oriColor]}/"

            printed = printed.rsplit("/", 1)
            printed = "".join(printed)
            printed += ")"

            return printed

    def WrongOpt(self, opt, obj="option"):
        print(f"{nr.ansi['error']}There is no such '{opt}' {obj}{nr.ansi['reset']}")

    def Warn(self):
        ans = (
            str(
                input(
                    f"{self.ansi['warn']}Warning: Once the process finished, the action can't be undone!! Are you sure you want to proceed? {self.opt('warn', 'yes', 'no')} {self.optX('warn', 'no')}"
                )
            )
            .lower()
            .strip()
        )
        print()
        return True if ans in ["yes", "y", "true", "t"] else False

    def getNamedRanges(self):
        # Retrieve the named ranges
        named_ranges = self.ss.list_named_ranges()
        nonr = len(named_ranges)
        print(f"{self.ansi['info']}In total, there {'are ' + str(nonr) + ' named ranges' if nonr > 1 else 'is 1 named range'}")
        if nonr:
            print(f"{self.ansi['info']}Example: {self.ansi['reset']}{named_ranges[0]}")
        print()
        return named_ranges

    def create(self):
        if self.Warn():
            named_ranges = self.getNamedRanges()
            named_range_names = [str(named_range["name"]) for named_range in named_ranges]

            worksheet = None
            while not worksheet:
                I = str(input(f"{self.ansi['info']}Input the worksheet name: {self.ansi['reset']}"))
                try:
                    worksheet = self.ss.worksheet(I)
                except Exception:
                    self.WrongOpt(I, "worksheet")
                    worksheet = None

            nonr = input(f"{self.ansi['info']}How many scenarios do you want to create the named ranges? (max: 999) {self.optX('info', '1')}")
            nonr = int(nonr) if nonr else 1
            nonr = nonr if nonr <= 999 else 999

            start = input(f"{self.ansi['info']}What is the first scenario? {self.optX('info', '1')}")
            start = int(start) if start else 1

            print(f"\n{self.ansi['info']}Processing: {self.ansi['reset']}")

            requests = []
            for i in range(start, nonr + start):
                iter_name = str(i).zfill(3)
                for j in ["Data", "Form"]:
                    name = f"_SN_{I.replace('-', '').replace('.', '_')}_Scenario_{iter_name}_{j}"
                    if name not in named_range_names:
                        print(f"{self.ansi['info']}- {name}{self.ansi['reset']}")
                        requests.append(
                            {
                                "addNamedRange": {
                                    "namedRange": {
                                        "name": name,
                                        "range": {
                                            "sheetId": worksheet.id,
                                            "startRowIndex": 0,
                                            "endRowIndex": 10,
                                            "startColumnIndex": 0,
                                            "endColumnIndex": 2,
                                        },
                                    }
                                }
                            }
                        )

            # Execute the batch update request
            if requests:
                self.ss.batch_update({"requests": requests})

            print(f"\n{self.ansi['success']}Creating finished with {len(requests)} has/have been processed. {self.ansi['reset']}")

    def delete(self):
        if self.Warn():
            named_ranges = self.getNamedRanges()
            named_range_names = [str(named_range["name"]) for named_range in named_ranges]
            opt = (
                str(
                    input(
                        f"{self.ansi['info']}Do you want to input the named ranges manually one by one or by words filter? {self.opt('info', 'manual', 'filter')} {self.optX('info', 'filter')}"
                    )
                )
                .lower()
                .strip()
            )
            opt = opt if opt else "filter"
            requests = []

            # Select the named ranges
            if opt == "manual":
                nrs = []
                while nr := str(input(f"{self.ansi['info']}Input the named range name (left it blank to stop inputting): {self.ansi['reset']}")).strip():
                    if nr in named_range_names:
                        nrs.append(nr)
                    else:
                        self.WrongOpt(nr, "named range")
                nrs = list(dict.fromkeys(nrs))
            elif opt == "filter":
                search = str(input(f"{self.ansi['info']}Input the word to be used for filtering: {self.ansi['reset']}")).lower().strip()
                nrs = list(filter(lambda x: search in x.lower(), named_range_names))
            else:
                self.WrongOpt(opt)
                return

            print(f"\n{self.ansi['info']}Processing: {self.ansi['reset']}")

            for nr in nrs:
                named_range = named_ranges[named_range_names.index(nr)]
                print(f"{self.ansi['info']}- {named_range['name']}{self.ansi['reset']}")
                request = {
                    "deleteNamedRange": {
                        "named_range_id": named_range["namedRangeId"],
                    },
                }
                requests.append(request)

            # Execute the batch update request
            if requests:
                self.ss.batch_update({"requests": requests})

            print(f"\n{self.ansi['success']}Deleting finished with {len(requests)} has/have been processed. {self.ansi['reset']}")

    def rename(self):
        if self.Warn():
            # Retrieve the named ranges
            named_ranges = self.getNamedRanges()
            named_range_names = [str(named_range["name"]) for named_range in named_ranges]

            # filter
            search = str(input(f"{self.ansi['info']}Input the word to be replaced: {self.ansi['reset']}"))
            replace = str(input(f"{self.ansi['info']}Input the replace: {self.ansi['reset']}"))
            nrs = list(filter(lambda x: search in x, named_range_names))

            print(f"\n{self.ansi['info']}Processing: {self.ansi['reset']}")

            # Iterate through the named ranges and update if needed
            requests = []
            for nr in nrs:
                named_range = named_ranges[named_range_names.index(nr)]
                print(f"{self.ansi['info']}- {named_range['name']}{self.ansi['reset']}")
                request = {
                    "updateNamedRange": {
                        "namedRange": {
                            "named_range_id": named_range["namedRangeId"],
                            "name": str(named_range["name"]).replace(search, replace),
                            "range": {
                                "sheetId": named_range["range"]["sheetId"],
                                "startRowIndex": named_range["range"]["startRowIndex"],
                                "endRowIndex": named_range["range"]["endRowIndex"],
                                "startColumnIndex": named_range["range"]["startColumnIndex"],
                                "endColumnIndex": named_range["range"]["endColumnIndex"],
                            },
                        }
                    }
                    | {"fields": "*"},
                }
                requests.append(request)

            # Execute the batch update request
            if requests:
                self.ss.batch_update({"requests": requests})

            print(f"\n{self.ansi['success']}Renaming finished with {len(requests)} has/have been processed. {self.ansi['reset']}")


if __name__ == "__main__":
    """
    This file is used to automatically handle named ranges, e.g. rename, and delete all named ranges automatically
    """

    nr = NamedRanges()

    # ask for the task option
    default_task = "create"
    inputted_task = str(input(f"{nr.ansi['info']}What task you want to do? {nr.opt('info', 'create', 'delete', 'rename')} {nr.optX('info', 'create')}")).lower().strip()
    task = inputted_task if inputted_task else default_task

    # do the task
    if task == "create":
        nr.create()
    elif task == "delete":
        nr.delete()
    elif task == "rename":
        nr.rename()
    else:
        nr.WrongOpt(task)



from decimal import Decimal
import datetime
import re


class Formatter:
    """Converting values into desired format result"""

    def convert_query_result(self, query_result, rounding_columns=None, rounding_option=None, toList=False):
        result = []

        # converting result to both decimal and datetime if it's possible
        for i in range(len(query_result)):
            for col in query_result[i]:
                query_result[i][col] = self.convert_decimal(query_result[i][col])
                query_result[i][col] = self.convert_datetime(query_result[i][col])
            result.append(query_result[i])

        # rounding number in result
        if rounding_columns:
            result = self.rounding(result, rounding_columns, rounding_option)

        # converting result into a single dimension list
        if toList:
            listResult = []
            for row in result:
                listResult += list(row.values())
            return listResult

        return result

    def convert_decimal(self, value):
        """Converts inputted value into decimal format if it is possible"""

        if isinstance(value, Decimal):
            return float(value)
        return value

    def convert_datetime(self, value):
        """Converts inputted value into desired date format if it is possible"""

        if isinstance(value, datetime.datetime):
            return value.strftime("%Y-%m-%d %H:%M:%S %Z")
        return value

    def convert_number(self, strNumber):
        """Converts inputted string into number if it's possible"""

        if len(strNumber):
            checkedStrNumber = strNumber.replace(".", "", 1)
            checkedStrNumber = checkedStrNumber.replace("-", "", 1) if checkedStrNumber[0] == "-" else checkedStrNumber
            if checkedStrNumber.isdigit():
                strNumber = round(float(strNumber), 1) if "." in strNumber else int(strNumber)
        return strNumber

    def rounding(self, query_result, column_names, option):
        """Iterats over query result and rounding all values in certain columns"""

        for row in query_result:
            for col in row:
                if col in column_names:
                    row[col] = round(row[col], option)
        return query_result

    def rgba_string_to_hex(self, rgba_string):
        """Converts RGBA string (mostly from Selenium) into hex code"""

        rgb_values = re.findall(r"\d+", rgba_string)
        r, g, b = map(int, rgb_values[:3])
        return "#{:02x}{:02x}{:02x}".format(r, g, b)

    def convert_period(self, period: str) -> list[list]:
        """
        Converts period into separated date

        Args:
            period (str): the period string

        Returns:
            list[list]: the separated date

        Example:
            >>>  convert_period("2023/06/15 - 2023/06/06")
            [[2023, 6, 15], [2023, 6, 6]]
        """

        return [[int(_) if _ != "" else _ for _ in date.split("/")] for date in period.split(" - ")]

    def re_sub(self, pattern, string):
        return re.sub(pattern, "", string).strip()



from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.firefox.webdriver import WebDriver
from selenium.webdriver.chrome.webdriver import WebDriver
from selenium.webdriver.edge.webdriver import WebDriver
from selenium.webdriver.common.by import By
import time

from .mouse_keys import MouseKeysDriver
from .wait import WaitDriver


class CheckDriver:
    def __init__(self, driver: WebDriver, duration: int) -> None:
        self.wd = WaitDriver(driver, duration)
        self.mkd = MouseKeysDriver(driver)
        self.__driver = driver

    def check_alert(self, isSuccess: bool, isVisible: bool, cust_message: str = None) -> bool:
        """
        Checks the presence of alert element

        Args:
            isSuccess (bool): tyoe of the alert, e.g. True means the alert is expected to be a 'success' type of alert
            isVisible (bool): alert condition, e.g. True means the alert is expected to be visible
            cust_message (str): is a custom message, which other than both 'success' and 'fail'

        Returns:
            bool: True means that the element is found, and vice versa
        """

        if isVisible:
            eAlert = self.wd.an_element(By.CLASS_NAME, "toast-body")
            if isSuccess:
                if eAlert.text == "success" or eAlert.text == cust_message:
                    return True
            else:
                if eAlert.text == "fail" or eAlert.text == cust_message:
                    return True
            return False
        else:
            eAlert = self.wd.invisible(By.CLASS_NAME, "toast-body")
            return True if eAlert else False

    def check_indicator_row(
        self,
        available_rows: list[WebElement],
        target_row: list,
    ) -> bool:
        """
        Checks whether an indicator row is listed or not

        Args:
            available_rows (list[WebElement]): contains all row elements
            target_row (list): contains the data to be compared

        Returns:
            bool: True means the result has met the expectation, and vice versa
        """

        def get_data_of_a_row(row: WebElement):
            tds = row.find_elements(By.TAG_NAME, "td")
            return [tds[i].text for i in range(4)]  # 4 are name, indicator, logic, and threshold value

        for row in available_rows:
            self.mkd.scrolling(element=row)

            rowdata = get_data_of_a_row(row)
            # check the current row
            if rowdata == target_row:
                return True
        return False

    def check_viewport(self, target_element: WebElement) -> bool:
        """
        Checks if an element is within the current viewport

        Args:
            target_element (WebElement): is the element whose position will be checked againts the viewport

        Returns:
            bool: True means that the element is within the viewport, and vice versa
        """

        if target_element:
            # since there are cases where the element is in progress to get into the viewport, e.g. scrolling, so then this loop is used to give such chances
            for i in range(7):
                # viewport attr
                viewport_height = self.__driver.execute_script("return window.innerHeight;")
                viewport_top = self.__driver.execute_script("return window.pageYOffset;")
                viewport_bottom = viewport_top + viewport_height

                # target element attr
                element_rect = target_element.rect
                element_top = element_rect["y"]
                element_bottom = element_top + element_rect["height"]

                if (viewport_top < element_top) and (viewport_bottom > element_bottom):
                    # The element is within the viewport
                    return True
                time.sleep(0.7)
        return False



import argparse
import shutil
import os


class StateSXT:
    def __init__(self) -> None:
        self.tree = [
            ".github",
            "base",
            "database",
            "testcases",
            "utils",
            ".env.template",
            ".gitignore",
            "named_ranges.py",
            "restate.py",
            "results.json",
            "pyproject.toml",
            "pytest.ini",
            "README.md",
            "tox.ini",
        ]
        self.deprs = {
            rf"testcases\_fixtures\login_fixture.py": rf"testcases\_fixtures\auth.py",
            rf"testcases\_fixtures\composition_fixture.py": rf"testcases\_fixtures\composition.py",
            rf"testcases\_fixtures\option_fixture.py": rf"testcases\_fixtures\option.py",
            rf"utils\explicit_wait.py": rf"utils\explicit.py",
            rf"utils\gsheet.py": rf"utils\service_account.py",
            rf".env-template": rf".env.template",
            rf"rename_named_ranges.py": rf"named_ranges.py",
            rf"execute_json.py": rf"restate.py",
            rf"retry.py": rf"restate.py",
            rf"track.json": rf"results.json",
            rf"last_run_data.json": rf"results.json",
            rf"testcases\fixtures": rf"testcases\_fixtures",
        }

        self.sourcedir = os.path.dirname(os.path.realpath(__file__))
        self.destdir = os.getcwd()
        self.destdirname = self.destdir.split("\\")[-1]
        self.ansi = {
            "error": "\033[91m\033[1m",
            "info": "\033[94m",
            "success": "\u001b[32m",
            "warn": "\u001b[33m",
            "bold": "\033[1m",
            "reset": "\033[0m",
            "underline": "\033[4m",
        }
        self.name = "StateSXT"
        self.optX = lambda oriColor, opt: f"[{self.ansi['success']}{opt}{self.ansi[oriColor]}]: {self.ansi['reset']}"
        self.opt1 = lambda oriColor, opt: f"({self.ansi['success']}{opt}{self.ansi[oriColor]})"
        self.opt2 = lambda oriColor, opt1, opt2: f"({self.ansi['success']}{opt1}{self.ansi[oriColor]}/{self.ansi['success']}{opt2}{self.ansi[oriColor]})"
        self.obj = lambda obj: f"'{obj}'"

    def toConfirm(self, input: input, adds: list[str] = []) -> bool:
        val = str(input).lower()
        res = True if (val in ["yes", "y"]) else False
        if adds:
            res = val if (val in adds) else res
        return res

    def warnVC(self) -> bool:
        print(
            f"\u2757{self.ansi['warn']} Before continuing the process, it is recommended that you have installed your project with a version control, e.g. Git/Github. This is to prevent your files/folders from being completely lost. {self.ansi['reset']}"
        )
        return self.toConfirm(input(f"\u2757{self.ansi['warn']} Do you want to proceed? {self.opt2('warn', 'yes', 'no')} {self.optX('warn', 'no')}"))

    def formatPath(self, path_name: str, isLower: bool = True, isName: bool = False) -> str:
        if isLower:
            path_name = path_name.lower()

        path_name = path_name.split("\\")[0].split("/")[0].strip()
        if not isName:
            path_name = path_name.replace(" ", "_")

        return path_name

    def getType(self, path: str, doWarn: bool = True) -> str | None:
        if any(type := [os.path.isdir(path), os.path.isfile(path)]):  # condition: when the folder/file already exists
            if type[0]:
                ptype = "folder"
            elif type[1]:
                ptype = "file"
            if doWarn:
                p = path.split("\\")[-1]
                print(f"{self.ansi['warn']}{ptype.capitalize()} {self.obj(p)} already exists{self.ansi['reset']}")
            return ptype

    def generate(self):
        print()
        rate = 0
        for p in self.tree:
            sourcepath = os.path.join(self.sourcedir, p)
            destpath = os.path.join(self.destdir, p)

            # generate the file/folder
            if self.getType(destpath):  # condition: when the folder/file already exists
                pass
            elif os.path.isdir(sourcepath):  # condition: when it's a folder
                shutil.copytree(sourcepath, destpath)
                rate += 1
            elif os.path.isfile(sourcepath):  # condition: when it's a file
                shutil.copy2(sourcepath, destpath)
                rate += 1
            else:  # condition: when the path does not exist
                print(f"{self.ansi['warn']}Path {self.obj(p)} does not exist{self.ansi['reset']}")

        # summary
        if rate == len(self.tree):
            print(f"{self.ansi['success']}All templates created in {self.obj(self.destdir)}{self.ansi['reset']}")
        elif rate == 1:
            print(f"\n{self.ansi['success']}A template created in {self.obj(self.destdir)}{self.ansi['warn']}, but {len(self.tree)-rate} failed.{self.ansi['reset']}")
        elif rate >= 1:
            print(f"\n{self.ansi['success']}{rate} templates created in {self.obj(self.destdir)}{self.ansi['warn']}, but {len(self.tree)-rate} failed.{self.ansi['reset']}")
        else:
            print(f"\n{self.ansi['error']}All templates failed to create in {self.obj(self.destdir)}{self.ansi['reset']}")

    def remove(self):
        isProceeded = self.warnVC()

        print()
        if isProceeded:
            rate = 0
            for p in self.tree:
                destpath = os.path.join(self.destdir, p)

                # remove the file/folder
                if os.path.isdir(destpath):  # condition: when it's a folder
                    shutil.rmtree(destpath)
                    rate += 1
                elif os.path.isfile(destpath):  # condition: when it's a file
                    os.remove(destpath)
                    rate += 1
                else:  # condition: when the path does not exist
                    print(f"{self.ansi['warn']}Path {self.obj(p)} does not exist{self.ansi['reset']}")

            # summary
            if rate == len(self.tree):
                print(f"{self.ansi['success']}All templates removed from {self.obj(self.destdir)}{self.ansi['reset']}")
            elif rate == 1:

                print(
                    f"\n{self.ansi['success']}{rate}A template has been removed from {self.obj(self.destdir)}{self.ansi['warn']}, but {len(self.tree)-rate} failed.{self.ansi['reset']}"
                )
            elif rate > 1:
                print(
                    f"\n{self.ansi['success']}{rate} templates have been removed from {self.obj(self.destdir)}{self.ansi['warn']}, but {len(self.tree)-rate} failed.{self.ansi['reset']}"
                )
            else:
                print(f"\n{self.ansi['error']}All templates failed to remove from {self.obj(self.destdir)}{self.ansi['reset']}")

    def update(self):
        isProceeded = self.warnVC()

        print()
        if isProceeded:
            # deep-loop over self.tree and gather the choices
            choices = []
            defaultSkippedSub = "C:\\"
            skippedSub = defaultSkippedSub
            for destpath, dirs, files in os.walk(self.destdir):
                sub = str(destpath).split(self.destdir + "\\")[-1] if (destpath != self.destdir) else ""
                subSplits = sub.split("\\")
                pathname = subSplits[-1]
                depth = len(subSplits) - 1

                if (skippedSub not in sub) or (skippedSub not in subSplits):
                    isRenamed = sub in self.deprs
                    if (os.path.exists(os.path.join(self.sourcedir, sub)) or isRenamed) and (
                        sub != ""
                    ):  # condition: check if the folder exists in source, and asking update to folder
                        updateOpt = self.toConfirm(
                            input(
                                " \u27b1 " * depth
                                + f"\u2753{self.ansi['info']} Do you want to update folder {self.obj(sub)} entirely? {self.opt2('info', 'yes', 'no')} or selectively? {self.opt1('info', 'select')} {self.optX('info', 'no')}"
                            ),
                            adds=["select"],
                        )
                        skippedSub = (
                            sub if (updateOpt != "select") else defaultSkippedSub
                        )  # if updateOpt is not 'select' then the next iteration will not check the files/folders within
                        if updateOpt == True:
                            choices.append([self.deprs[sub] if isRenamed else sub, "folder", sub if isRenamed else None])

                    if (skippedSub not in sub) and (skippedSub not in subSplits):  # condition: when updateOpt tells to update folder entirely
                        for file in files:  # condition: loop over files within destpath, and asking update to files
                            pathname = file
                            pSub = os.path.join(sub, pathname)
                            psubSplits = pSub.split("\\")
                            depth = len(psubSplits) - 1
                            isRenamed = pSub in self.deprs
                            if os.path.exists(os.path.join(self.sourcedir, pSub)) or isRenamed:  # condition: check if the file exists in source
                                updateOpt = self.toConfirm(
                                    input(
                                        " \u27b1 " * depth
                                        + f"\u2753{self.ansi['info']} Do you want to update file {self.obj(pSub)}? {self.opt2('info', 'yes', 'no')} {self.optX('info', 'no')}"
                                    ),
                                )
                                if updateOpt:
                                    choices.append([self.deprs[pSub] if isRenamed else pSub, "file", pSub if isRenamed else None])

            print()
            # loop over the choices and update
            for c in choices:
                sourcepath = os.path.join(self.sourcedir, c[0])
                destpath = os.path.join(self.destdir, c[0])

                if c[1] == "folder":
                    try:
                        shutil.rmtree(os.path.join(self.destdir, c[2]) if c[2] else destpath)
                        shutil.copytree(sourcepath, destpath)
                        print(f"{self.ansi['success']}Folder {self.obj(c[0])} updated successfully{self.ansi['reset']}")
                    except Exception as e:
                        print(f"{self.ansi['error']}Folder {self.obj(c[0])} failed to update{self.ansi['reset']}")
                elif c[1] == "file":
                    try:
                        os.remove(os.path.join(self.destdir, c[2]) if c[2] else destpath)
                        shutil.copy2(sourcepath, destpath)
                        print(f"{self.ansi['success']}File {self.obj(c[0])} updated successfully{self.ansi['reset']}")
                    except Exception as e:
                        print(f"{self.ansi['error']}File {self.obj(c[0])} failed to update{self.ansi['reset']}")
                else:
                    print(f"{self.ansi['error']}Type is invalid{self.ansi['reset']}")

    def createPage(self):
        nameX = "example"  # default name
        nameI = self.formatPath(  # input name
            str(input(f"\U0001F4D3{self.ansi['info']} Page name {self.optX('info', nameX)}")),
            isLower=False,
            isName=True,
        )
        page_name = nameI if nameI else nameX

        print()
        parent_folder = "testcases"
        if os.path.exists(os.path.join(self.destdir, parent_folder)):  # condition: when folder /testcases exists
            page_name_path = self.formatPath(page_name)
            if not os.path.exists(os.path.join(self.destdir, f"{parent_folder}\{page_name_path}")):  # condition: when folder with name page_name does not exist
                template_folder = os.path.join(self.sourcedir, "template")
                destpath = os.path.join(self.destdir, f"{parent_folder}\{page_name_path}")

                # Copy the template folder to the destination
                shutil.copytree(template_folder, destpath)

                # Rename files inside the copied folder and modify their content
                for root, dirs, files in os.walk(destpath):
                    if "__pycache__" in root:  # condition: skip the __pycache__ folder and its contents
                        continue

                    # loop over files in the dir, and change the contents
                    for name in files:
                        file_path = os.path.join(root, name)
                        with open(file_path, "r", encoding="utf-8") as f:
                            content = f.read()
                        content = content.replace("example", page_name.lower().replace(" ", "_"))
                        content = content.replace("Example", page_name.title().replace(" ", ""))
                        content = content.replace("EXAMPLE", page_name.upper().replace(" ", ""))
                        with open(file_path, "w", encoding="utf-8") as f:
                            f.write(content)

                print(f"{self.ansi['success']}New page template created in {self.obj(self.destdir)}{self.ansi['reset']}")
            else:
                print(f"{self.ansi['error']}A page folder with name {self.obj(page_name_path)} already exists{self.ansi['reset']}")
        else:
            print(f"{self.ansi['error']}{self.name} could not find folder {self.obj(parent_folder)} in {self.obj(self.destdir)}{self.ansi['reset']}")

    def cli(self):
        parser = argparse.ArgumentParser(description="Generate Directories")
        parser.add_argument("opt", help="Action to perform: 'generate', 'remove', 'update', and 'create-page'", choices=["generate", "remove", "update", "create-page"])
        parser.add_argument("-v", "--version", action="version", version="StateSXT 0.5.5")
        args = parser.parse_args()

        if str(args.opt).lower() == "generate":
            self.generate()
        elif str(args.opt).lower() == "remove":
            self.remove()
        elif str(args.opt).lower() == "update":
            self.update()
        elif str(args.opt).lower() == "create-page":
            self.createPage()
        else:
            print("StateSXT does not has such command.")


def main():
    StateSXT().cli()


if __name__ == "__main__":
    main()



import json


class Store:
    def __init__(self) -> None:
        self.__results = {"Scenario": {}, "Test Cases": {}}
        self.ansiPassed = "\u001b[32m"
        self.ansiFailed = "\033[91m\033[1m"
        self.ansiReset = "\033[0m"
        self.notifPassed = f"{self.ansiPassed}PASSED{self.ansiReset}"
        self.notifFailed = f"{self.ansiFailed}FAILED{self.ansiReset}"
        self.json_path = "results.json"

    def getResults(self):
        return self.__results

    def printUseCaseResults(self, class_name):
        print(f"\n\nTest Results of {class_name}")
        classResults = self.__results["Scenario"][class_name]
        for res in classResults:
            print(f"- {res}: {self.notifPassed if classResults[res] == 'PASSED' else self.notifFailed}")

    def setTestCasesResult(self, worksheet_name, named_range, result):
        try:
            self.__results["Test Cases"][worksheet_name][named_range] = result
        except:
            self.__results["Test Cases"][worksheet_name] = {named_range: result}

    def setScenarioResult(self, class_name, scenario, result):
        try:
            self.__results["Scenario"][class_name][scenario] = result
        except:
            self.__results["Scenario"][class_name] = {scenario: result}

    def updateJSON(self):
        try:
            with open(self.json_path, "w") as json_file:
                json.dump(self.__results, json_file, indent=4)
        except Exception as e:
            print(f"An error happened when trying to save the result data to {self.json_path}: \n{e}")



import pytest


def pytest_addoption(parser):
    parser.addoption("--browser", "-B")
    parser.addoption("--domain", "-D")
    parser.addoption("--report")
    parser.addoption("--tfo")
    parser.addoption("--use_email")
    parser.addoption(
        "--number-help",
        action="store_true",
        default=False,
        help="Print custom number help information and exit.",
    )


@pytest.fixture(scope="session")
def browser(request):
    req = request.config.getoption("--browser") or request.config.getoption("-B")
    return req if req else "edge"


@pytest.fixture(scope="session")
def domain(request):
    req = request.config.getoption("--domain") or request.config.getoption("-D")
    return req if req else None


@pytest.fixture(scope="session")
def report(request):
    req = request.config.getoption("--report")
    return req if req else "0"


@pytest.fixture(scope="session")
def tfo(request):
    req = request.config.getoption("--tfo")
    return req if req else "1"


@pytest.fixture(scope="session")
def use_email(request):
    req = request.config.getoption("--use_email")
    return req if req else "0"


def pytest_collection_modifyitems(config, items):
    if config.option.number_help:
        print(
            """
        Browser:
        - 1 = brave
        - 2 = chrome
        - 3 = edge
        - 4 = firefox

        Domain:
        - None
        """
        )
        items.clear()



import pytest
import os

from utils.email import Email
from utils.service_account import ServiceAccount
from utils.logger import Logger
from utils.store import Store


@pytest.fixture(scope="session")
def email(use_email):
    yield
    email = Email(
        sender_email=os.getenv("SENDER_EMAIL"),
        sender_password=os.getenv("SENDER_PASSWORD"),
        receiver_email=os.getenv("RECEIVER_EMAIL").split(","),
        receiver_name=os.getenv("RECEIVER_NAME").split(","),
    )
    if use_email == "1":
        try:
            print("\n\nSending email...")
            email.send()
            print("Email has been sent successfully")
        except Exception as e:
            print(f"Email failed to send: {str(e)}")


@pytest.fixture(scope="session")
def logger():
    lg = Logger()
    yield
    lg.shutdown()


@pytest.fixture(scope="session")
def service_account(report, tfo, domain):
    sa = ServiceAccount(
        spreadsheetName=os.getenv("SPREADSHEET_NAME"),
        folderId=os.getenv("FOLDER_ID"),
        domain=domain,
        testedFilesOnly=False if (tfo == "0") else True,
    )
    yield sa
    if report == "1":
        print("\n\nUpdating gsheet...")
        sa.update_all_values()
        sa.update_worksheet_colors()


@pytest.fixture(scope="session")
def store():
    store = Store()
    yield store



import logging
import pytest
import json
import sys

from testcases.top_menu.page import TopMenuPage
from testcases.login.page import LoginPage


accounts = json.load(open("./database/mocks/mock_login_valid.json"))


@pytest.fixture(scope="function")
def auth(request):
    try:
        LoginPage(request.cls.base).login(
            accounts[request.cls.role]["username"],
            accounts[request.cls.role]["password"],
        )
        yield
        TopMenuPage(request.cls.base).logout()
    except Exception as e:
        logging.getLogger(f"root.{__name__}.{sys._getframe().f_code.co_name}").error(f"login error:\n{str(e)}")
        raise Exception(str(e))



from cryptography.fernet import Fernet


class Crypter:
    """Simply to encrypt and decrypt"""

    def decrypt(self, key, encryptedMessage):
        return key.decrypt(encryptedMessage).decode()

    def encrypt(self, key, message):
        return key.encrypt(message.encode())

    def generateKey(self):
        return Fernet(Fernet.generate_key())



from .. import Locator


class ExampleLocator(Locator):
    """Example page locator class"""

    def __init__(self, base) -> None:
        super().__init__(base)
        self.setup()

    def setup(self):
        # flags
        self.EXAMPLE_LOCATOR1 = lambda loc="exampleloc1": self.bd.wd.clickable(self.by.xpath, loc)
        self.EXAMPLE_LOCATOR2 = lambda loc="exampleloc2": self.bd.wd.visible(self.by.xpath, loc)



from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.firefox.webdriver import WebDriver
from selenium.webdriver.chrome.webdriver import WebDriver
from selenium.webdriver.edge.webdriver import WebDriver
from selenium.webdriver.common.keys import Keys
from typing import Callable
from typing import Union
import time as t


class MouseKeysDriver:
    """
    Provides Mouse and Keys capabilities.
    """

    def __init__(self, driver: WebDriver) -> None:
        self.ac = ActionChains(driver)
        self.__driver = driver

    def clicking(
        self,
        element: Union[WebElement, Callable[[], WebElement]],
        isClick: bool = True,
        sleep: float = 0.36,
        steps: int = None,
        block: str = "center",
        useJS: bool = True,
    ) -> None:
        if isClick:
            if isinstance(element, Callable):
                element = element()
            if sleep >= 1:
                self.scrolling(
                    element=element,
                    steps=steps,
                    block=block,
                    sleep=sleep,
                )
            # click the element
            (
                self.__driver.execute_script(
                    "arguments[0].click();",
                    element,
                )
                if useJS
                else element.click()
            )

    def hovering(
        self,
        element: Union[WebElement, Callable[[], WebElement]],
        isHover: bool = True,
        onFocus: bool = False,
    ) -> None:
        if isHover:
            if isinstance(element, Callable):
                element = element()
            if onFocus:
                self.scrolling(element)
            self.ac.move_to_element(element).perform()
            self.ac.reset_actions()

    def paginating(
        self,
        target_row: list[str],
        func_get_paginations: Callable[[], list[WebElement]],
        func_check_rows: Callable[[list[str]], bool],
        direction: str = "backward",
        tobeFound: bool = True,
        sleep: float = 0,
    ):
        def get_list_of_pagination_numbers():
            raw_pns = func_get_paginations()
            pns = []
            titles = []
            for pn in raw_pns:
                title = pn.get_attribute("title")
                if title in [
                    "previous page",
                    "next page",
                    "first page",
                    "last page",
                ]:
                    pns.append(pn)
                    titles.append(title)
            return [pns, titles]

        def click_page_number(title: str, pns: list[WebElement], titles: list[str]):
            """
            - clicks the title page and updates the pagination numbers, and their titles
            - title is whether 'previous page','next page','first page', or 'last page'
            """

            pn = pns[titles.index(title)]
            self.scrolling(element=pn)
            pn.click()
            return get_list_of_pagination_numbers()

        pns, titles = get_list_of_pagination_numbers()
        if direction == "backward":
            # go to the last page
            if "last page" in titles:
                pns, titles = click_page_number(title="last page", pns=pns, titles=titles)
            else:
                # backwarding (without checking)
                while "next page" in titles:
                    pns, titles = click_page_number(title="next page", pns=pns, titles=titles)

        # checking while also backwarding
        occur = 0
        while occur < 1:
            # check whether or not the condition is ready to stop
            if ("previous page" not in titles) if (direction == "backward") else ("next page" not in titles):  # when in page item-1
                occur += 1
            # check current page's rows
            t.sleep(sleep)
            if func_check_rows(target_row):
                return True if tobeFound else False
            # forwarding / backwarding (depends on the direction)
            if occur != 1:
                pns, titles = (
                    click_page_number(title="previous page", pns=pns, titles=titles) if (direction == "backward") else click_page_number(title="next page", pns=pns, titles=titles)
                )
        return False if tobeFound else True

    def scrolling(
        self,
        element: Union[WebElement, Callable[[], WebElement]] = None,
        steps: int = None,
        block: str = "center",
        sleep: float = 0.75,
    ) -> None:
        """
        'block', defines vertical alignment.
        - 'start'
        - 'center' (default)
        - 'end'
        - 'nearest'
        """

        if element:
            if isinstance(element, Callable):
                element = element()
            if steps:
                self.ac.scroll_by_amount(0, steps).pause(sleep).perform()
                self.__driver.execute_script(
                    "arguments[0].scrollIntoView();",
                    element,
                )
            else:
                self.__driver.execute_script(
                    "arguments[0].scrollIntoView({block: '" + block + "'});",
                    element,
                )
                t.sleep(sleep)
        else:
            self.ac.scroll_by_amount(0, steps).pause(sleep).perform()
            self.ac.reset_actions()

    def zooming(self, zoom_percentage: float, sleep: int = 1) -> None:
        """
        Zooms in and out the screen.

        Args:
            - zoom_percentage (float), is the percentage of the screen wanted to be
        """
        self.__driver.execute_script(f"document.body.style.zoom = '{zoom_percentage/100}';")
        t.sleep(sleep)



from googleapiclient.discovery import build
from datetime import datetime as dt
import gspread
import json


class ServiceAccount:
    """Implementing a service account capabilities"""

    def __init__(
        self,
        spreadsheetName,
        folderId,
        domain: str = None,
        testedFilesOnly=True,
    ) -> None:
        self.__folderId = folderId
        self.__newSs = None
        self.__sa = gspread.service_account()
        self.__ss = self.__sa.open(spreadsheetName)
        self.automationName = "Selenium"
        self.curDate = dt.now().strftime("%Y/%m/%d %H:%M:%S")
        self.json_path = "results.json"
        self.newSpreadsheetName = f"Automation - Release {self.curDate} - {domain.upper()}" if domain else f"Automation - Release {self.curDate}"
        self.testedFilesOnly = testedFilesOnly

    @property
    def sa(self):
        return self.__sa

    @property
    def ss(self):
        return self.__ss

    def create_a_copy_of_gsheet_file(self):
        self.sa.copy(file_id=self.ss.id, title=self.newSpreadsheetName, copy_permissions=True)
        self.__newSs = self.sa.open(self.newSpreadsheetName)
        if self.testedFilesOnly:
            deleteRequests = []
            initialSheets = ["Cover", "Use Cases", "ToC", "Queries", "variables"]
            for wks in self.__newSs.worksheets():
                if wks.title not in initialSheets:
                    deleteRequests.append({"deleteSheet": {"sheetId": wks.id}})
            self.__newSs.batch_update({"requests": deleteRequests})

    def create_a_copy_of_worksheet_into_new_gsheet_file_and_update_the_values(self, worksheetName, namedRange, values):
        try:  # assuming that the gsheet has already a worksheet with paramater name
            wks = self.__newSs.worksheet(worksheetName)
        except:  # assuming that the gsheet does not have any worksheet the same with the parameter
            oldWks = self.ss.worksheet(worksheetName)
            wks = self.__newSs.worksheet(oldWks.copy_to(self.__newSs.id)["title"])
            wks.update_title(worksheetName)
        wks.update(namedRange, values, value_input_option="USER_ENTERED")

    def get_json(self):
        with open(self.json_path, "r") as json_file:
            return json.load(json_file)["Test Cases"]

    def get_values_by_named_range(self, worksheetName, namedRange):
        wks = self.ss.worksheet(worksheetName)
        return wks.get(namedRange)

    def update_all_values(self):
        data = self.get_json()
        if data:
            # create a new file (the duplicate of the target file)
            self.create_a_copy_of_gsheet_file()

            for worksheetName in data:
                for namedRange in data[worksheetName]:
                    values = [
                        [
                            self.curDate,
                            self.automationName,
                            internalCheckResult,
                            externalCheckResult,
                            testerNote,
                        ]
                        for internalCheckResult, externalCheckResult, testerNote in data[worksheetName][namedRange]
                    ]
                    self.create_a_copy_of_worksheet_into_new_gsheet_file_and_update_the_values(worksheetName, namedRange.replace("Data", "Form"), values)
            # remove sheet1, which is the default sheet that is created when creating a new gsheet file
            if self.__newSs.sheet1.title == "Sheet1":
                self.__newSs.del_worksheet(self.__newSs.sheet1)
            self.upload_the_gsheet_file_to_folder()
        else:
            print("No test result when updating all values in Google Sheet")

    def update_worksheet_colors(self):
        data = self.get_json()
        if data:
            for wksName in data:
                wksId = self.__newSs.worksheet(wksName).id
                noFail = True
                for nr in data[wksName]:
                    if len(list(filter(lambda x: x[0] == "FAILED", data[wksName][nr]))):
                        noFail = False
                        break

                if noFail:
                    requestsBatch = [
                        {
                            "updateSheetProperties": {
                                "properties": {
                                    "sheetId": wksId,
                                    "tabColor": {
                                        "red": 0.0,  # Specify the color values in RGB format (from 0.0 to 1.0)
                                        "green": 1.0,
                                        "blue": 0.0,
                                    },
                                },
                                "fields": "tabColor",
                            }
                        }
                    ]
                else:
                    requestsBatch = [
                        {
                            "updateSheetProperties": {
                                "properties": {
                                    "sheetId": wksId,
                                    "tabColor": {
                                        "red": 1.0,  # Specify the color values in RGB format (from 0.0 to 1.0)
                                        "green": 0.0,
                                        "blue": 0.0,
                                    },
                                },
                                "fields": "tabColor",
                            }
                        }
                    ]

                # Send the batchUpdate request
                self.__newSs.batch_update({"requests": requestsBatch})
        else:
            print("No test result when updating worksheet colors in Google Sheet")

    def upload_the_gsheet_file_to_folder(self):
        # Move the newly created spreadsheet to the desired folder
        drive_service = build("drive", "v3", credentials=self.sa.auth)
        drive_service.files().update(fileId=self.__newSs.id, addParents=self.__folderId, fields="id,parents").execute()



from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.firefox.webdriver import WebDriver
from selenium.webdriver.chrome.webdriver import WebDriver
from selenium.webdriver.edge.webdriver import WebDriver
from selenium.webdriver.support.ui import Select
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.by import By
from datetime import datetime
from typing import Callable
from typing import Union

from .mouse_keys import MouseKeysDriver
from utils.formatter import Formatter
from .wait import WaitDriver


class FormDriver:
    def __init__(self, driver: WebDriver, duration: int) -> None:
        self.ac = ActionChains(driver)
        self.mkd = MouseKeysDriver(driver)
        self.wd = WaitDriver(driver, duration)

    def check_a_box(self, element: WebElement, isChecked: bool):
        if element.is_selected():
            None if isChecked else element.click()
        else:
            element.click() if isChecked else None

    def get_selected_option(
        self,
        element: WebElement,
    ):
        return Select(element).first_selected_option

    def insert_to_textbox(
        self,
        element: Union[WebElement, Callable[[], WebElement]],
        input: str,
        byEnter: bool = False,
        sleep: float = 0.5,
        onFocus: bool = False,
    ) -> None:
        """
        Inserts string into a textbox element

        Args:
            element (WebElement): is the input element
            input (str): is the string to be inputted
            byEnter (bool): is the final action, e.g. True means the enter key will be pressed
            sleep: duration between action
            onFocus: to scroll before inserting

        Returns:
            None
        """

        if input or (input == ""):
            if isinstance(element, Callable):
                element = element()
            if onFocus:
                self.mkd.scrolling(element)

            self.ac.pause(sleep).click(element).send_keys(Keys.END).key_down(Keys.SHIFT).send_keys(Keys.HOME).key_up(Keys.SHIFT).send_keys(Keys.BACKSPACE).send_keys(
                input
            ).perform()

            if byEnter:
                self.ac.pause(sleep).send_keys(Keys.ENTER).perform()

            self.ac.reset_actions()

            # for faster approach
            # textbox_element.click()
            # textbox_element.clear()
            # textbox_element.send_keys(input)
            # textbox_element.send_keys(Keys.ENTER)

    def select_opt_in_dropdown(
        self,
        element: Union[WebElement, Callable[[], WebElement]],
        option,
        method="visible_text",
        onFocus: bool = False,
    ):
        if option:
            if isinstance(element, Callable):
                element = element()
            if onFocus:
                self.mkd.scrolling(element)

            select = Select(element)
            if method == "value":
                select.select_by_value(option)
            elif method == "visible_text":
                select.select_by_visible_text(option)

    def select_opt_in_radio(
        self,
        elements: Union[list[WebElement], Callable[[], list[WebElement]]],
        option: str,
    ):
        if isinstance(elements, Callable):
            elements = elements()
        clickables, menus = [
            [c for c in elements[::2]],
            [str(m.text).lower() for m in elements[1::2]],
        ]
        index = menus.index(option.lower())
        clickables[index].click()

    def select_period(
        self,
        period: str,
        date_input_element: WebElement,
        prev_button_locator,
        next_button_locator,
        date_header_locator,
        date_per_week_locator,
        date_per_day_locator,
        onFocus: bool = False,
    ):
        """
        Input value into the period element

        Args:
            period (str): is the date/period
            input (str): is the string to be inputted
            byEnter (bool): is the final action, e.g. True means the enter key will be pressed

        Returns:
            None
        """

        if period:
            if onFocus:
                self.mkd.scrolling(date_input_element)

            start_date: list[int]
            end_date: list[int]
            start_date, end_date = Formatter().convert_period(period)

            # click the period date element
            date_input_element.click()

            # provide prev and next month button
            click_prev = lambda: self.wd.wdw.until(lambda d: date_input_element.find_element(By.XPATH, prev_button_locator)).click()
            click_next = lambda: self.wd.wdw.until(lambda d: date_input_element.find_element(By.XPATH, next_button_locator)).click()

            def prev_and_next(num_of_clicks: int):
                if num_of_clicks > 0:
                    for _ in range(num_of_clicks):
                        click_next()
                else:
                    for _ in range(abs(num_of_clicks)):
                        click_prev()

            def select_date(selected_date: str):
                week_elements = self.wd.all_elements(By.CLASS_NAME, date_per_week_locator)
                day_elements = self.wd.all_elements(By.CLASS_NAME, date_per_day_locator)
                day_elements_per_week = []
                for i in range(len(week_elements)):
                    el = day_elements[(i * 7) : ((i + 1) * 7)]
                    day_elements_per_week.append(el)

                isFound = False
                num_of_weeks = len(day_elements_per_week)
                for i in range(num_of_weeks):
                    for date in day_elements_per_week[i]:
                        if (i == 0 and int(date.text) > 7) or (i == num_of_weeks and int(date.text) < 20):
                            continue
                        if date.text == selected_date:
                            date.click()
                            isFound = True
                            break
                    if isFound:
                        break

            # get current date
            date_header = self.wd.wdw.until(lambda d: date_input_element.find_element(By.XPATH, date_header_locator)).text.split()
            cur_month = int(datetime.strptime(date_header[0], "%B").month)
            cur_year = int(date_header[1])

            # get number of clicks needed to reach the start_date
            month_diff_with_curr = start_date[1] - cur_month
            year_diff_with_curr = start_date[0] - cur_year
            start_clicks = 0
            start_clicks += month_diff_with_curr
            start_clicks += year_diff_with_curr * 12

            # move to the month and year of the start_date
            prev_and_next(start_clicks)

            # select date of the start_date
            select_date(selected_date=str(start_date[2]))

            if len(end_date) == 3:
                # get number of clicks needed to reach the end_date
                month_diff_with_end_date = end_date[1] - start_date[1]
                year_diff_with_end_date = end_date[0] - start_date[0]
                end_clicks = 0
                end_clicks += month_diff_with_end_date
                end_clicks += year_diff_with_end_date * 12

                # move to the month and year of the end_date
                prev_and_next(end_clicks)

                # select date of the end_date
                select_date(selected_date=str(end_date[2]))

            date_input_element.click()



from selenium.webdriver.firefox.service import Service as FirefoxService
from selenium.webdriver.chrome.service import Service as ChromeService
from selenium.webdriver.chrome.service import Service as BraveService
from selenium.webdriver.edge.service import Service as EdgeService
from webdriver_manager.microsoft import EdgeChromiumDriverManager
from webdriver_manager.core.os_manager import ChromeType
from webdriver_manager.chrome import ChromeDriverManager
from webdriver_manager.firefox import GeckoDriverManager
from selenium import webdriver
import logging
import time
import sys

from .mouse_keys import MouseKeysDriver
from .check import CheckDriver
from .table import TableDriver
from .form import FormDriver
from .wait import WaitDriver


class BaseDriver:
    """Wraps driver and provides all common-used WebDriver actions"""

    def __init__(self, browser, domain=None, fullscreen=True, duration: int = 10) -> None:
        self.setup(browser, domain, fullscreen)

        self.cd = CheckDriver(self.__driver, duration)
        self.fd = FormDriver(self.__driver, duration)
        self.mkd = MouseKeysDriver(self.__driver)
        self.td = TableDriver(self.__driver, duration)
        self.wd = WaitDriver(self.__driver, duration)

    def setup(self, browser, domain, fullscreen) -> None:
        """Sets up which browser and domain to use"""

        try:
            # setup driver
            print("\nSetting up driver...", end=" ")
            start = time.time()
            if browser == "brave":
                self.__driver = webdriver.Chrome(service=BraveService(ChromeDriverManager(chrome_type=ChromeType.BRAVE).install()))
            elif browser == "chrome":
                self.__driver = webdriver.Chrome(service=ChromeService(ChromeDriverManager().install()))
            elif browser == "edge":
                self.__driver = webdriver.Edge(service=EdgeService(EdgeChromiumDriverManager().install()))
            elif browser == "firefox":
                self.__driver = webdriver.Firefox(service=FirefoxService(GeckoDriverManager().install()))
            end = time.time()
            print(f"Takes {end-start} seconds to complete.")

            # setup domain
            print("\nSetting up domain...", end=" ")
            start = time.time()
            if domain:
                self.__driver.get("<Your Custom Domain URL>")
            else:
                self.__driver.get("<Your URL>")
            end = time.time()
            print(f"Takes {end-start} seconds to complete.")

            # setup screen size
            if fullscreen:
                self.__driver.maximize_window()
        except Exception as e:
            logging.getLogger(f"root.{__name__}.{self.__class__.__name__}.{sys._getframe().f_code.co_name}").error(f"in the process of running setup:\n{str(e)}")
            raise Exception(str(e))

    def close_tab(self) -> None:
        """Closes the current tab, and backs to the initial tab"""

        self.__driver.close()
        self.focus_to(0)

    def exit(self) -> None:
        self.__driver.quit()

    def focus_to(self, index: int = -1) -> None:
        """Changes the focus of the driver"""

        self.__driver.switch_to.window(self.__driver.window_handles[index])

    def go_to(self, href: str):
        self.__driver.get(href)

    def navigate(self, url: str) -> None:
        """Loads a web page in the current browser session"""

        self.__driver.get(url)
        self.focus_to()



from pretty_html_table import build_table
from email.message import EmailMessage
from abc import ABC, abstractmethod
import pandas as pd
import smtplib
import ssl


class Email:
    """Class to send all emails related to scheduler test cases"""

    context = ssl.create_default_context()
    emailSubject = ""
    emailBody = None
    num_of_successes = None
    num_of_fails = None

    @abstractmethod
    def send(self):
        pass

    def __init__(self, sender_email, sender_password, receiver_email, receiver_name, results={}) -> None:
        self.results = results
        self.__senderEmail = sender_email
        self.__senderPassword = sender_password
        self.__receiverEmail = receiver_email
        self.__receiverName = receiver_name

    def calculateFailsSuccesses(self):
        self.num_of_successes = len(list(filter(lambda x: self.results[x][-1] == "PASSED", self.results)))
        self.num_of_fails = len(self.results) - self.num_of_successes

    def generateEmailBody(self, receiverName):
        self.emailBody = f"""
            Hi {receiverName}-san,
            <br><br>
            I hope this email finds you well. Attached table is the Selenium Test Automation Report, providing an overview of the test results and performance for our software application.
            <br><br>
            Below is a summary table highlighting the key metrics from the Selenium test suite:
            {
                build_table(
                    pd.DataFrame.from_dict(self.results),
                    "blue_dark"
                )
            }
            Thank you for your attention to this matter.
            <br><br>
            Best regards,<br>
            QA-Team
        """

    def generateEmailSubject(self):
        self.calculateFailsSuccesses()
        if self.num_of_successes > 1 and self.num_of_fails > 1:
            self.emailSubject = f"{self.emailSubject} - There were {self.num_of_successes} successes and {self.num_of_fails} failures"
        elif self.num_of_successes > 1 and self.num_of_fails <= 1:
            self.emailSubject = f"{self.emailSubject} - There were {self.num_of_successes} successes and {self.num_of_fails} failure"
        elif self.num_of_successes <= 1 and self.num_of_fails > 1:
            self.emailSubject = f"{self.emailSubject} - There were {self.num_of_successes} success and {self.num_of_fails} failures"
        else:
            self.emailSubject = f"{self.emailSubject} - There were {self.num_of_successes} success and {self.num_of_fails} failure"

    def send(self):
        if len(self.__receiverName) == len(self.__receiverEmail):
            for receiverName, receiverEmail in zip(self.__receiverName, self.__receiverEmail):
                # create an email object
                lib = EmailMessage()

                # generate the email body and email subject
                self.generateEmailSubject()
                self.generateEmailBody(receiverName)

                # set up email
                lib["From"] = self.__senderEmail
                lib["To"] = receiverEmail
                lib["Subject"] = self.emailSubject
                lib.add_alternative(self.emailBody, subtype="html")

                # send email
                with smtplib.SMTP_SSL("smtp.gmail.com", 465, context=self.context) as smtp:
                    smtp.login(self.__senderEmail, self.__senderPassword)
                    smtp.send_message(lib)
                    smtp.close()
        else:
            raise Exception("number of receiverName does not fit with the number of receiverEmail")



from typing import Callable
import time as t


def explicit(
    condition: Callable,
    timeout: int = 9,
    poll_frequency: float = 0.5,
    returnBool: bool = False,
):
    start = t.time()
    while (t.time() - start) < timeout:
        if condition():
            return True if returnBool else None
        t.sleep(poll_frequency)
    if returnBool:
        return False
    raise TimeoutError("Condition not met within the specified timeout.")



from .states.ts01 import ExampleInitState
from .locator import ExampleLocator
from .. import Page


class ExamplePage(Page):
    """Example Page action methods"""

    def __init__(self, base):
        super().__init__(base)
        self.initState = ExampleInitState(base, self)
        self.state = self.initState
        self.lr = ExampleLocator(base)

    """
    Methods: Abstract
    """

    # region
    def changeState(self, newState):
        self.state = newState

    def resetState(self):
        self.state = self.initState

    # endregion

    """
    Methods: Interface
    """

    # region ==> is a Python feature to freely collapsing code, ended with endregion
    def exampleTransition(self, exampleParam):
        return self.state.exampleTransition(exampleParam)

    # endregion

    """
    Methods: Specific
    """
    # region
    # endregion



from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.by import By

from utils.wrapper import Wrapper


class MyBy:
    """
    Provides keys that each links to an 'By' object.
    The purpose is to have control to By objects name.

    For example:
    If we want to rename one of the keys, that can be done by renaming it just in here.
    Of course do not forget to use F2 before renaming it, so it applies to all references.
    """

    xpath = By.XPATH
    css = By.CSS_SELECTOR
    cname = By.CLASS_NAME
    id = By.ID
    link = By.LINK_TEXT
    plink = By.PARTIAL_LINK_TEXT
    name = By.NAME
    tag = By.TAG_NAME


class WaitDriver:
    """
    Provides explicit wait, i.e. will wait until either the element has found or exceed time limit.
    """

    def __init__(self, driver, duration: int) -> None:
        self.init_duration = duration
        self.wdw = WebDriverWait(driver, duration)

    @Wrapper.exception_handling_returns_None
    def all_elements(self, by: MyBy, locator, cusdur: float = 0, cusfreq: float = 0.5) -> list[WebElement]:
        # config
        if cusdur > 0:
            self.wdw._timeout = cusdur
        if cusfreq != 0.5:
            self.wdw._poll = cusfreq
        # get
        res = self.wdw.until(EC.presence_of_all_elements_located((by, locator)))
        # reset
        self.wdw._timeout = self.init_duration
        self.wdw._poll = 0.5
        return res

    @Wrapper.exception_handling_returns_None
    def an_element(self, by: MyBy, locator, cusdur: float = 0, cusfreq: float = 0.5) -> WebElement:
        # config
        if cusdur > 0:
            self.wdw._timeout = cusdur
        if cusfreq != 0.5:
            self.wdw._poll = cusfreq
        # get
        res = self.wdw.until(EC.presence_of_element_located((by, locator)))
        # reset
        self.wdw._timeout = self.init_duration
        self.wdw._poll = 0.5
        return res

    @Wrapper.exception_handling_returns_None
    def clickable(self, by: MyBy, locator, cusdur: float = 0, cusfreq: float = 0.5) -> WebElement:
        # config
        if cusdur > 0:
            self.wdw._timeout = cusdur
        if cusfreq != 0.5:
            self.wdw._poll = cusfreq
        # get
        res = self.wdw.until(EC.element_to_be_clickable((by, locator)))
        # reset
        self.wdw._timeout = self.init_duration
        self.wdw._poll = 0.5
        return res

    @Wrapper.exception_handling_returns_None
    def invisible(self, by: MyBy, locator, cusdur: float = 0, cusfreq: float = 0.5) -> bool:
        # config
        if cusdur > 0:
            self.wdw._timeout = cusdur
        if cusfreq != 0.5:
            self.wdw._poll = cusfreq
        # get
        res = self.wdw.until(EC.invisibility_of_element_located((by, locator)))
        # reset
        self.wdw._timeout = self.init_duration
        self.wdw._poll = 0.5
        return res

    @Wrapper.exception_handling_returns_None
    def visible(self, by: MyBy, locator, cusdur: float = 0, cusfreq: float = 0.5) -> WebElement:
        # config
        if cusdur > 0:
            self.wdw._timeout = cusdur
        if cusfreq != 0.5:
            self.wdw._poll = cusfreq
        # get
        res = self.wdw.until(EC.visibility_of_element_located((by, locator)))
        # reset
        self.wdw._timeout = self.init_duration
        self.wdw._poll = 0.5
        return res



from seleniumwire.utils import decode
from seleniumwire import webdriver
import json


class ResponseHandler:
    """To get response of calls (making use selenium-wire)"""

    def get_response(self, driver: webdriver, prefix=""):
        data = []
        for request in driver.requests:
            if request.response:
                if request.url.startswith(prefix):
                    response = request.response
                    body = decode(
                        response.body,
                        response.headers.get("Content-Encoding", "identity"),
                    )
                    decoded_body = body.decode("utf-8")
                    json_data = json.loads(decoded_body)
                    data.append(json_data)
        return data



class Query:
    """
    As an endpoint to interact with database
    """

    """Queries"""
    __GET_ALL_EXAMPLES = """
        SELECT * FROM EXAMPLE_TABLE
    """

    """GETTERS"""

    @property
    def __GET_ALL_EXAMPLES(self):
        return self.__GET_ALL_EXAMPLES



from .. import StateInterface

from abc import ABC
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .page import ExamplePage


class ExampleInterface(StateInterface, ABC):
    def __init__(self, base, contextPage: "ExamplePage") -> None:
        super().__init__(base)
        self.p = contextPage

    def exampleTransition(self, *args, **kwargs):
        pass



import pytest

from utils.wrapper import Wrapper
from . import TestExample


@pytest.mark.dev
class TestExample01(TestExample):
    def __init__(self, methodName: str = "runTest"):
        super().__init__(methodName)

    @Wrapper.result_receiving
    @Wrapper.unpagshe(*("0.1", "_SN_0_1_Scenario_001_Data"))
    def test_scenario001(self, *args):
        """Test Scenario: 1-1"""

        # an example of transition (1-1)
        self.soft_assert(self.assertIsNone, self.p.exampleTransition(exampleParam=args[0]))

        self.assert_all()



from abc import ABC, abstractmethod
from dotenv import load_dotenv
import warnings

from base.base_driver import BaseDriver
from base.wait import MyBy

# looding all env variables and ignoring a certain warning
load_dotenv()
warnings.filterwarnings("ignore", message="Worksheet.update.*method signature will change.*")


class StateInterface(ABC):
    """
    An abstract class for all the state interface classes
    """

    def __init__(self, base: BaseDriver) -> None:
        self.__bd = base

    @property
    def bd(self):
        return self.__bd


class Page(ABC):
    """A parent class of all specific page classes."""

    def __init__(self, base: BaseDriver) -> None:
        self.__bd = base
        self.jpnFormats = ["jpn", "japan", "japanese", "jp"]
        self.engFormats = ["eng", "english", "en"]
        self.emptyFormats = ["", "-", "<blank>", "<empty>", "blank", "empty"]
        self.anyFormats = ["anything", "dc", "Any", "any"]
        self.spaceFormats = ["<space>"]

    @property
    def bd(self):
        return self.__bd

    @abstractmethod
    def changeState(self):
        pass

    @abstractmethod
    def resetState(self):
        pass


class Locator:
    """A parent class of all page locator classes."""

    def __init__(self, base: BaseDriver) -> None:
        self.__bd = base
        self.by = MyBy()

    @property
    def bd(self):
        return self.__bd



import logging


class RootFilter(logging.Filter):
    def __init__(self, name: str) -> None:
        super().__init__(name)
        self.name = name

    def filter(self, record: logging.LogRecord) -> bool:
        return record.name.startswith(self.name)


class Logger:
    """Logging messages for a specific system or application component"""

    def __init__(self) -> None:
        # instantiate logging components
        self.logger = logging.getLogger("root")
        self.file_handler = logging.FileHandler("automation.log", mode="w")
        # self.console_handler = logging.StreamHandler()
        self.formatter = logging.Formatter("%(asctime)s | %(name)s | %(levelname)s | %(message)s")

        # set up
        self.setup()

    def setup(self):
        # set level of the logger
        self.logger.setLevel(logging.ERROR)

        # set level of the handler
        self.file_handler.setLevel(logging.ERROR)
        # self.console_handler.setLevel(logging.ERROR)

        # install formatter into the handlers
        self.file_handler.setFormatter(self.formatter)
        # self.console_handler.setFormatter(self.formatter)

        # add filters
        self.logger.addFilter(RootFilter(name=self.logger.name))
        self.file_handler.addFilter(RootFilter(name=self.logger.name))
        # self.console_handler.addFilter(RootFilter(name="root"))

        # add the handlers to the logger
        self.logger.addHandler(self.file_handler)
        # self.logger.addHandler(self.console_handler)

    def shutdown(self):
        logging.shutdown()



from functools import wraps
import logging
import sys

from .faker import FakerGenerator


class Wrapper:
    """Making use functools\wraps"""

    @classmethod
    def exception_handling_returns_None(cls, func):
        """
        to let a test case returns a None value instead of raises an exception/error
        """
        decoratorClassName = cls.__name__
        decoratorMethodName = sys._getframe().f_code.co_name

        @wraps(func)
        def wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as e:
                logging.getLogger(f"root.{__name__}.{decoratorClassName}.{decoratorMethodName}").error(f"error:\n{str(e)}")
                return None

        return wrapper

    @classmethod
    def exception_handling_raises_error(cls, func):
        """
        to handle the error by tracking, but keeps raises the error
        """
        decoratorClassName = cls.__name__
        decoratorMethodName = sys._getframe().f_code.co_name

        @wraps(func)
        def wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as e:
                logging.getLogger(f"root.{__name__}.{decoratorClassName}.{decoratorMethodName}").error(f"error:\n{str(e)}")
                raise Exception(str(e))

        return wrapper

    @classmethod
    def result_receiving(cls, func):
        """
        to track the result of test cases, so instead of directly raising error, it lets to write down the error first, e.g. email, report, and summary
        """
        decoratorClassName = cls.__name__
        decoratorMethodName = sys._getframe().f_code.co_name

        @wraps(func)
        def wrapper(self, *args, **kwargs):
            funcName = str(func.__name__).replace("_", " ").title()
            className = self.__class__.__name__
            isFail = False

            try:
                func(self, *args, **kwargs)
                self.store.setScenarioResult(className, funcName, "PASSED")
            except Exception as e:
                if str(e).replace("'", "") != funcName:
                    logging.getLogger(f"root.{__name__}.{decoratorClassName}.{decoratorMethodName}").error(f"class: {self.__class__.__name__}, method: {func.__name__}\n{str(e)}")
                self.store.setScenarioResult(className, funcName, "FAILED")
                errorMsg = str(e)
                isFail = True

            self.store.printUseCaseResults(className)
            self.store.updateJSON()
            if isFail:
                raise Exception(errorMsg)

        return wrapper

    @classmethod
    def unpagshe(cls, worksheet, named_range, needExternalCheck=False):
        """
        to retrieve and unpack the data from gsheet
        """
        decoratorClassName = cls.__name__
        decoratorMethodName = sys._getframe().f_code.co_name

        def decorator(func):
            @wraps(func)
            def wrapper(self, *args, **kwargs):
                data = self.sa.get_values_by_named_range(worksheet, named_range)
                result = []
                isFail = False
                emptyFormats = [
                    "",
                    "-",
                    "<blank>",
                    "<empty>",
                    "blank",
                    "empty",
                    "inactive",
                    "uncheck",
                ]
                anyFormats = ["anything", "dc", "Any", "any"]

                for row in data:
                    # preprocess data
                    row = [None if (col in emptyFormats) else col for col in row]
                    row = [(FakerGenerator().generate_sentence() if (col in anyFormats) else col) for col in row]

                    try:
                        func(self, *row, *args, **kwargs)
                        result.append(["PASSED", "PASSED" if needExternalCheck else "", ""])
                        self.p.resetState()
                    except Exception as e:
                        logging.getLogger(f"root.{__name__}.{decoratorClassName}.{decoratorMethodName}").error(
                            f"class: {self.__class__.__name__}, method: {func.__name__}\n{str(e)}"
                        )
                        # raise Exception(str(e))
                        result.append(["FAILED", "FAILED" if needExternalCheck else "", f"'{str(e)}'"])
                        errorMsg = str(e)
                        isFail = True

                self.store.setTestCasesResult(worksheet, named_range, result)
                if isFail:
                    raise Exception(errorMsg)

            return wrapper

        return decorator



from database.database import ExampleDB
from database.queries import Query
from utils.formatter import Formatter
import os


class DBService:
    """Database service"""

    def __init__(self) -> None:
        self.setup()

        self.__query = Query()
        self.ft = Formatter()

    def setup(self):
        SSH_HOST = os.getenv("POSTGRE_DB_SSH_HOST")
        SSH_PORT = os.getenv("POSTGRE_DB_SSH_PORT")
        SSH_USERNAME = os.getenv("POSTGRE_DB_SSH_USERNAME")
        SSH_PASSWORD = os.getenv("POSTGRE_DB_SSH_PASSWORD")
        DB_SERVER = os.getenv("POSTGRE_DB_DB_SERVER")
        DB_PORT = os.getenv("POSTGRE_DB_DB_PORT")
        EXAMPLE_DB_NAME = os.getenv("EXAMPLE_DB_DB_NAME")
        EXAMPLE_DB_USER = os.getenv("EXAMPLE_DB_DB_USER")
        EXAMPLE_DB_PASSWORD = os.getenv("EXAMPLE_DB_DB_PASSWORD")
        self.__exampleDB = ExampleDB(
            DB_NAME=EXAMPLE_DB_NAME,
            DB_USER=EXAMPLE_DB_USER,
            DB_PASSWORD=EXAMPLE_DB_PASSWORD,
            SSH_HOST=SSH_HOST,
            SSH_PORT=SSH_PORT,
            SSH_USERNAME=SSH_USERNAME,
            SSH_PASSWORD=SSH_PASSWORD,
            DB_SERVER=DB_SERVER,
            DB_PORT=DB_PORT,
        )

    def start(self):
        self.__exampleDB.connect()

    def end(self):
        self.__exampleDB.disconnect()

    # example page
    def get_all_examples(self):
        qResult = self.__exampleDB.query_read(self.__query.__GET_ALL_EXAMPLES)
        return self.ft.convert_query_result(
            query_result=qResult,
            rounding_columns=[],
            rounding_option=1,
        )



from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.common.by import By

from utils.formatter import Formatter
from .wait import WaitDriver


class TableDriver:
    """
    Provides the capability to interact with tables.
    """

    def __init__(self, driver, duration: int) -> None:
        self.ft = Formatter()
        self.ac = ActionChains(driver)
        self.wd = WaitDriver(driver, duration)

    def get_attribute(self, element, option=[]):
        if "color" in option:
            if element.text == "":
                return None
            return self.ft.rgba_string_to_hex(element.value_of_css_property("color"))

    def get_data_from_table(
        self,
        table: WebElement,
        hint_table=None,
        hint_header=False,
        column_names=[],
        attribute=[],
        ignored_table_columns_to_hover=[],
        ignored_table_rows_to_hover=[],
        ignored_hint_columns=[],
    ):
        rows = self.get_table_rows(table)
        data = []
        for i in range(len(rows)):
            cols = self.get_table_cols(rows[i])
            temp = {}
            if i not in ignored_table_rows_to_hover:
                for j in range(len(cols)):
                    if (hint_table) and (j not in ignored_table_columns_to_hover):
                        temp[column_names[j]] = [self.ft.convert_number(s.replace("%", "").strip("'")) for s in repr(cols[j].text).split("\\n")] + [
                            self.get_data_on_hint(cols[j], hint_table, hint_header, ignored_hint_columns)
                        ]
                        continue
                    if j in attribute:
                        temp[column_names[j]] = [self.ft.convert_number(s.replace("%", "").strip("'")) if (s != "''") else None for s in repr(cols[j].text).split("\\n")] + [
                            self.get_attribute(cols[j], option=["color"])
                        ]
                        continue
                    temp[column_names[j]] = [self.ft.convert_number(s.replace("%", "").strip("'")) if (s != "''") else None for s in repr(cols[j].text).split("\\n")]
                data.append(temp)

        return data

    def get_data_on_hint(self, hoverable_element, hint_table, hint_header, ignored_columns):
        self.ac.move_to_element(hoverable_element).perform()
        rows = self.wd.all_elements(By.XPATH, hint_table)
        result = []
        for row in rows if hint_header else rows[1:]:
            cols = row.find_elements(By.TAG_NAME, "div")
            result.append([self.ft.convert_number(cols[i].text) for i in range(len(cols)) if i not in ignored_columns])
        return result

    def get_table_cols(self, row: WebElement):
        return row.find_elements(By.TAG_NAME, "td")

    def get_table_rows(self, table: WebElement):
        tbody = table.find_element(By.TAG_NAME, "tbody")
        return tbody.find_elements(By.TAG_NAME, "tr")



from ..interface import ExampleInterface


class ExampleInitState(ExampleInterface):
    def __init__(self, base, contextPage) -> None:
        super().__init__(base, contextPage)

    def exampleTransition(self, exampleParam):
        """Required Process"""
        self.bd.fd.insert_to_textbox(self.p.lr.EXAMPLE_LOCATOR1(), exampleParam)
        """Transition"""
        self.p.changeState(ExampleInitState(self.bd, self.p))



from sshtunnel import SSHTunnelForwarder
import psycopg2


class PostgreDB:
    """Root database"""

    def __init__(self, SSH_HOST, SSH_PORT, SSH_USERNAME, SSH_PASSWORD, DB_SERVER, DB_PORT) -> None:
        self.__SSH_HOST = SSH_HOST
        self.__SSH_PORT = int(SSH_PORT)
        self.__SSH_USERNAME = SSH_USERNAME
        self.__SSH_PASSWORD = SSH_PASSWORD
        self.__DB_SERVER = DB_SERVER
        self.__DB_PORT = int(DB_PORT)
        self.__DB_NAME = ""
        self.__DB_USER = ""
        self.__DB_PASSWORD = ""

    """GETTERS"""

    @property
    def SSH_HOST(self):
        return self.__SSH_HOST

    @property
    def SSH_PORT(self):
        return self.__SSH_PORT

    @property
    def SSH_USERNAME(self):
        return self.__SSH_USERNAME

    @property
    def SSH_PASSWORD(self):
        return self.__SSH_PASSWORD

    @property
    def DB_SERVER(self):
        return self.__DB_SERVER

    @property
    def DB_PORT(self):
        return self.__DB_PORT

    @property
    def DB_NAME(self):
        return self.__DB_NAME

    @property
    def DB_USER(self):
        return self.__DB_USER

    @property
    def DB_PASSWORD(self):
        return self.__DB_PASSWORD

    def connect(self):
        try:
            with SSHTunnelForwarder(
                (self.SSH_HOST, self.SSH_PORT),
                ssh_username=self.SSH_USERNAME,
                ssh_password=self.SSH_PASSWORD,
                remote_bind_address=(self.DB_SERVER, self.DB_PORT),
                local_bind_address=("xxx.x.x.x", 0),
            ) as server:
                server.start()
                params = {
                    "database": self.DB_NAME,
                    "user": self.DB_USER,
                    "password": self.DB_PASSWORD,
                    # "host": self.DB_SERVER,
                    "host": "127.0.0.1",
                    # "port": self.DB_PORT,
                    "port": server.local_bind_port,
                }
                self.conn = psycopg2.connect(**params)
                self.cursor = self.conn.cursor()

        except Exception as e:
            print(f"\n\nerror message: {str(e)}\n\n")
            print(str(e))
            raise Exception("Connection Failed")

    def disconnect(self):
        self.conn.close()

    def query_read(self, query):
        try:
            self.cursor.execute(query)
            column_names = [desc[0] for desc in self.cursor.description]
            rows = self.cursor.fetchall()
            results = []
            for row in rows:
                result = dict(zip(column_names, row))
                results.append(result)
            return results
        except Exception as e:
            self.conn.rollback()
            raise ConnectionError(str(e))

    def query_add_edit(self, query):
        self.cursor.execute(query)
        self.conn.commit()
        return self.cursor.rowcount


class ExampleDB(PostgreDB):
    """Example database"""

    def __init__(
        self,
        DB_NAME,
        DB_USER,
        DB_PASSWORD,
        SSH_HOST,
        SSH_PORT,
        SSH_USERNAME,
        SSH_PASSWORD,
        DB_SERVER,
        DB_PORT,
    ) -> None:
        super().__init__(SSH_HOST, SSH_PORT, SSH_USERNAME, SSH_PASSWORD, DB_SERVER, DB_PORT)
        self.__DB_NAME = DB_NAME
        self.__DB_USER = DB_USER
        self.__DB_PASSWORD = DB_PASSWORD

    """GETTERS"""

    @property
    def DB_NAME(self):
        return self.__DB_NAME

    @property
    def DB_USER(self):
        return self.__DB_USER

    @property
    def DB_PASSWORD(self):
        return self.__DB_PASSWORD



from utils.service_account import ServiceAccount

from dotenv import load_dotenv
import os


def run():
    sa = ServiceAccount(
        spreadsheetName=os.getenv("SPREADSHEET_NAME"),
        folderId=os.getenv("FOLDER_ID"),
        testedFilesOnly=True,
    )
    print("Updating gsheet with json...")
    sa.update_all_values()
    sa.update_worksheet_colors()
    print("Updating complete!")


if __name__ == "__main__":
    """
    Basically this file is purposed to execute the results.json file (located on root), which contains the test result of the last test execution. This action is taken regarding to avoid 2 main problems, i.e. request limit and unexpected error during the gsheet upgrade process.
    """

    load_dotenv()
    run()



import setuptools

with open("README.md", encoding="utf8") as file:
    LONG_DESC = file.read()

setuptools.setup(
    name="statesxt",
    version="0.5.5",
    description="A project template for testing your website application.",
    long_description=LONG_DESC,
    long_description_content_type="text/markdown",
    url="https://test.pypi.org/project/statesxt/",
    author="Jason Caleb",
    author_email="cjsonnnnn@gmail.com",
    license="MIT License",
    project_urls={
        "Source": "https://github.com/jsonnnnn/statesxt",
        "Documentation": "https://statesxt.readthedocs.io/en/latest/",
    },
    classifiers=[
        "Development Status :: 1 - Planning",
        "Framework :: Pytest",
        "Intended Audience :: Developers",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python",
        "Topic :: Software Development :: Testing",
        "Topic :: Utilities",
    ],
    entry_points={
        "console_scripts": [
            "statesxt=statesxt.main:main",
        ],
    },
    keywords=[
        "Selenium",
        "Pytest",
        "Python",
        "project template",
        "template",
        "Testing",
        "Framework",
    ],
    python_requires=">=3.10",
    packages=setuptools.find_packages(),
    include_package_data=True,
    setup_requires=["setuptools-git"],
)



import pytest
import softest

from ..page import ExamplePage


@pytest.mark.order(1)
@pytest.mark.usefixtures("setup")
class TestExample(softest.TestCase):
    """Test cases for example page"""

    @pytest.fixture(autouse=True)
    def class_setup(self):
        self.p = ExamplePage(self.base)



import pytest

"""
Set Role
"""
# region


@pytest.fixture(scope="class")
def admin(request):
    request.cls.role = "admin"


@pytest.fixture(scope="class")
def manager(request):
    request.cls.role = "manager"


@pytest.fixture(scope="class")
def operator(request):
    request.cls.role = "operator"


@pytest.fixture(scope="class")
def supervisor(request):
    request.cls.role = "supervisor"


# endregion


"""
Set Use Case
"""


# region
@pytest.fixture(scope="class")
def usecase1(request):
    request.cls.usecase = 1


@pytest.fixture(scope="class")
def usecase2(request):
    request.cls.usecase = 2


@pytest.fixture(scope="class")
def usecase3(request):
    request.cls.usecase = 3


@pytest.fixture(scope="class")
def usecase5(request):
    request.cls.usecase = 4


@pytest.fixture(scope="class")
def usecase5(request):
    request.cls.usecase = 5


@pytest.fixture(scope="class")
def usecase6(request):
    request.cls.usecase = 6


@pytest.fixture(scope="class")
def usecase7(request):
    request.cls.usecase = 7


# endregion



from faker import Faker


class FakerGenerator:
    """Generating fake values"""

    def __init__(self) -> None:
        self.faker = Faker()

    def generate_name(self):
        return self.faker.name()

    def generate_sentence(self, num_of_sentences: int = 1):
        return self.faker.paragraph(nb_sentences=num_of_sentences)

    def generate_title(self):
        return self.faker.company()
